// For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT

pub const sparcConst = enum(c_int) {

// SPARC32 CPU

	CPU_SPARC32_FUJITSU_MB86904 = 0,
	CPU_SPARC32_FUJITSU_MB86907 = 1,
	CPU_SPARC32_TI_MICROSPARC_I = 2,
	CPU_SPARC32_TI_MICROSPARC_II = 3,
	CPU_SPARC32_TI_MICROSPARC_IIEP = 4,
	CPU_SPARC32_TI_SUPERSPARC_40 = 5,
	CPU_SPARC32_TI_SUPERSPARC_50 = 6,
	CPU_SPARC32_TI_SUPERSPARC_51 = 7,
	CPU_SPARC32_TI_SUPERSPARC_60 = 8,
	CPU_SPARC32_TI_SUPERSPARC_61 = 9,
	CPU_SPARC32_TI_SUPERSPARC_II = 10,
	CPU_SPARC32_LEON2 = 11,
	CPU_SPARC32_LEON3 = 12,
	CPU_SPARC32_ENDING = 13,

// SPARC64 CPU

	CPU_SPARC64_FUJITSU = 0,
	CPU_SPARC64_FUJITSU_III = 1,
	CPU_SPARC64_FUJITSU_IV = 2,
	CPU_SPARC64_FUJITSU_V = 3,
	CPU_SPARC64_TI_ULTRASPARC_I = 4,
	CPU_SPARC64_TI_ULTRASPARC_II = 5,
	CPU_SPARC64_TI_ULTRASPARC_III = 6,
	CPU_SPARC64_TI_ULTRASPARC_IIE = 7,
	CPU_SPARC64_SUN_ULTRASPARC_III = 8,
	CPU_SPARC64_SUN_ULTRASPARC_III_CU = 9,
	CPU_SPARC64_SUN_ULTRASPARC_IIII = 10,
	CPU_SPARC64_SUN_ULTRASPARC_IV = 11,
	CPU_SPARC64_SUN_ULTRASPARC_IV_PLUS = 12,
	CPU_SPARC64_SUN_ULTRASPARC_IIII_PLUS = 13,
	CPU_SPARC64_SUN_ULTRASPARC_T1 = 14,
	CPU_SPARC64_SUN_ULTRASPARC_T2 = 15,
	CPU_SPARC64_NEC_ULTRASPARC_I = 16,
	CPU_SPARC64_ENDING = 17,

// SPARC registers

	SPARC_REG_INVALID = 0,
	SPARC_REG_F0 = 1,
	SPARC_REG_F1 = 2,
	SPARC_REG_F2 = 3,
	SPARC_REG_F3 = 4,
	SPARC_REG_F4 = 5,
	SPARC_REG_F5 = 6,
	SPARC_REG_F6 = 7,
	SPARC_REG_F7 = 8,
	SPARC_REG_F8 = 9,
	SPARC_REG_F9 = 10,
	SPARC_REG_F10 = 11,
	SPARC_REG_F11 = 12,
	SPARC_REG_F12 = 13,
	SPARC_REG_F13 = 14,
	SPARC_REG_F14 = 15,
	SPARC_REG_F15 = 16,
	SPARC_REG_F16 = 17,
	SPARC_REG_F17 = 18,
	SPARC_REG_F18 = 19,
	SPARC_REG_F19 = 20,
	SPARC_REG_F20 = 21,
	SPARC_REG_F21 = 22,
	SPARC_REG_F22 = 23,
	SPARC_REG_F23 = 24,
	SPARC_REG_F24 = 25,
	SPARC_REG_F25 = 26,
	SPARC_REG_F26 = 27,
	SPARC_REG_F27 = 28,
	SPARC_REG_F28 = 29,
	SPARC_REG_F29 = 30,
	SPARC_REG_F30 = 31,
	SPARC_REG_F31 = 32,
	SPARC_REG_F32 = 33,
	SPARC_REG_F34 = 34,
	SPARC_REG_F36 = 35,
	SPARC_REG_F38 = 36,
	SPARC_REG_F40 = 37,
	SPARC_REG_F42 = 38,
	SPARC_REG_F44 = 39,
	SPARC_REG_F46 = 40,
	SPARC_REG_F48 = 41,
	SPARC_REG_F50 = 42,
	SPARC_REG_F52 = 43,
	SPARC_REG_F54 = 44,
	SPARC_REG_F56 = 45,
	SPARC_REG_F58 = 46,
	SPARC_REG_F60 = 47,
	SPARC_REG_F62 = 48,
	SPARC_REG_FCC0 = 49,
	SPARC_REG_FCC1 = 50,
	SPARC_REG_FCC2 = 51,
	SPARC_REG_FCC3 = 52,
	SPARC_REG_G0 = 53,
	SPARC_REG_G1 = 54,
	SPARC_REG_G2 = 55,
	SPARC_REG_G3 = 56,
	SPARC_REG_G4 = 57,
	SPARC_REG_G5 = 58,
	SPARC_REG_G6 = 59,
	SPARC_REG_G7 = 60,
	SPARC_REG_I0 = 61,
	SPARC_REG_I1 = 62,
	SPARC_REG_I2 = 63,
	SPARC_REG_I3 = 64,
	SPARC_REG_I4 = 65,
	SPARC_REG_I5 = 66,
	SPARC_REG_FP = 67,
	SPARC_REG_I7 = 68,
	SPARC_REG_ICC = 69,
	SPARC_REG_L0 = 70,
	SPARC_REG_L1 = 71,
	SPARC_REG_L2 = 72,
	SPARC_REG_L3 = 73,
	SPARC_REG_L4 = 74,
	SPARC_REG_L5 = 75,
	SPARC_REG_L6 = 76,
	SPARC_REG_L7 = 77,
	SPARC_REG_O0 = 78,
	SPARC_REG_O1 = 79,
	SPARC_REG_O2 = 80,
	SPARC_REG_O3 = 81,
	SPARC_REG_O4 = 82,
	SPARC_REG_O5 = 83,
	SPARC_REG_SP = 84,
	SPARC_REG_O7 = 85,
	SPARC_REG_Y = 86,
	SPARC_REG_XCC = 87,
	SPARC_REG_PC = 88,
	SPARC_REG_PSR = 89,
	SPARC_REG_ENDING = 90,
	SPARC_REG_O6 = 84,
	SPARC_REG_I6 = 67,

};
