// For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT

pub const m68kConst = enum(c_int) {

// M68K CPU

	CPU_M68K_M5206 = 0,
	CPU_M68K_M68000 = 1,
	CPU_M68K_M68020 = 2,
	CPU_M68K_M68030 = 3,
	CPU_M68K_M68040 = 4,
	CPU_M68K_M68060 = 5,
	CPU_M68K_M5208 = 6,
	CPU_M68K_CFV4E = 7,
	CPU_M68K_ANY = 8,
	CPU_M68K_ENDING = 9,

// M68K registers

	M68K_REG_INVALID = 0,
	M68K_REG_A0 = 1,
	M68K_REG_A1 = 2,
	M68K_REG_A2 = 3,
	M68K_REG_A3 = 4,
	M68K_REG_A4 = 5,
	M68K_REG_A5 = 6,
	M68K_REG_A6 = 7,
	M68K_REG_A7 = 8,
	M68K_REG_D0 = 9,
	M68K_REG_D1 = 10,
	M68K_REG_D2 = 11,
	M68K_REG_D3 = 12,
	M68K_REG_D4 = 13,
	M68K_REG_D5 = 14,
	M68K_REG_D6 = 15,
	M68K_REG_D7 = 16,
	M68K_REG_SR = 17,
	M68K_REG_PC = 18,
	M68K_REG_CR_SFC = 19,
	M68K_REG_CR_DFC = 20,
	M68K_REG_CR_VBR = 21,
	M68K_REG_CR_CACR = 22,
	M68K_REG_CR_TC = 23,
	M68K_REG_CR_MMUSR = 24,
	M68K_REG_CR_SRP = 25,
	M68K_REG_CR_USP = 26,
	M68K_REG_CR_MSP = 27,
	M68K_REG_CR_ISP = 28,
	M68K_REG_CR_URP = 29,
	M68K_REG_CR_ITT0 = 30,
	M68K_REG_CR_ITT1 = 31,
	M68K_REG_CR_DTT0 = 32,
	M68K_REG_CR_DTT1 = 33,
	M68K_REG_ENDING = 34,

};
