// For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT

package unicorn;

public interface SparcConst {

    // SPARC32 CPU

    public static final int UC_CPU_SPARC32_FUJITSU_MB86904 = 0;
    public static final int UC_CPU_SPARC32_FUJITSU_MB86907 = 1;
    public static final int UC_CPU_SPARC32_TI_MICROSPARC_I = 2;
    public static final int UC_CPU_SPARC32_TI_MICROSPARC_II = 3;
    public static final int UC_CPU_SPARC32_TI_MICROSPARC_IIEP = 4;
    public static final int UC_CPU_SPARC32_TI_SUPERSPARC_40 = 5;
    public static final int UC_CPU_SPARC32_TI_SUPERSPARC_50 = 6;
    public static final int UC_CPU_SPARC32_TI_SUPERSPARC_51 = 7;
    public static final int UC_CPU_SPARC32_TI_SUPERSPARC_60 = 8;
    public static final int UC_CPU_SPARC32_TI_SUPERSPARC_61 = 9;
    public static final int UC_CPU_SPARC32_TI_SUPERSPARC_II = 10;
    public static final int UC_CPU_SPARC32_LEON2 = 11;
    public static final int UC_CPU_SPARC32_LEON3 = 12;
    public static final int UC_CPU_SPARC32_ENDING = 13;

    // SPARC64 CPU

    public static final int UC_CPU_SPARC64_FUJITSU = 0;
    public static final int UC_CPU_SPARC64_FUJITSU_III = 1;
    public static final int UC_CPU_SPARC64_FUJITSU_IV = 2;
    public static final int UC_CPU_SPARC64_FUJITSU_V = 3;
    public static final int UC_CPU_SPARC64_TI_ULTRASPARC_I = 4;
    public static final int UC_CPU_SPARC64_TI_ULTRASPARC_II = 5;
    public static final int UC_CPU_SPARC64_TI_ULTRASPARC_III = 6;
    public static final int UC_CPU_SPARC64_TI_ULTRASPARC_IIE = 7;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_III = 8;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_III_CU = 9;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_IIII = 10;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_IV = 11;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_IV_PLUS = 12;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_IIII_PLUS = 13;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_T1 = 14;
    public static final int UC_CPU_SPARC64_SUN_ULTRASPARC_T2 = 15;
    public static final int UC_CPU_SPARC64_NEC_ULTRASPARC_I = 16;
    public static final int UC_CPU_SPARC64_ENDING = 17;

    // SPARC registers

    public static final int UC_SPARC_REG_INVALID = 0;
    public static final int UC_SPARC_REG_F0 = 1;
    public static final int UC_SPARC_REG_F1 = 2;
    public static final int UC_SPARC_REG_F2 = 3;
    public static final int UC_SPARC_REG_F3 = 4;
    public static final int UC_SPARC_REG_F4 = 5;
    public static final int UC_SPARC_REG_F5 = 6;
    public static final int UC_SPARC_REG_F6 = 7;
    public static final int UC_SPARC_REG_F7 = 8;
    public static final int UC_SPARC_REG_F8 = 9;
    public static final int UC_SPARC_REG_F9 = 10;
    public static final int UC_SPARC_REG_F10 = 11;
    public static final int UC_SPARC_REG_F11 = 12;
    public static final int UC_SPARC_REG_F12 = 13;
    public static final int UC_SPARC_REG_F13 = 14;
    public static final int UC_SPARC_REG_F14 = 15;
    public static final int UC_SPARC_REG_F15 = 16;
    public static final int UC_SPARC_REG_F16 = 17;
    public static final int UC_SPARC_REG_F17 = 18;
    public static final int UC_SPARC_REG_F18 = 19;
    public static final int UC_SPARC_REG_F19 = 20;
    public static final int UC_SPARC_REG_F20 = 21;
    public static final int UC_SPARC_REG_F21 = 22;
    public static final int UC_SPARC_REG_F22 = 23;
    public static final int UC_SPARC_REG_F23 = 24;
    public static final int UC_SPARC_REG_F24 = 25;
    public static final int UC_SPARC_REG_F25 = 26;
    public static final int UC_SPARC_REG_F26 = 27;
    public static final int UC_SPARC_REG_F27 = 28;
    public static final int UC_SPARC_REG_F28 = 29;
    public static final int UC_SPARC_REG_F29 = 30;
    public static final int UC_SPARC_REG_F30 = 31;
    public static final int UC_SPARC_REG_F31 = 32;
    public static final int UC_SPARC_REG_F32 = 33;
    public static final int UC_SPARC_REG_F34 = 34;
    public static final int UC_SPARC_REG_F36 = 35;
    public static final int UC_SPARC_REG_F38 = 36;
    public static final int UC_SPARC_REG_F40 = 37;
    public static final int UC_SPARC_REG_F42 = 38;
    public static final int UC_SPARC_REG_F44 = 39;
    public static final int UC_SPARC_REG_F46 = 40;
    public static final int UC_SPARC_REG_F48 = 41;
    public static final int UC_SPARC_REG_F50 = 42;
    public static final int UC_SPARC_REG_F52 = 43;
    public static final int UC_SPARC_REG_F54 = 44;
    public static final int UC_SPARC_REG_F56 = 45;
    public static final int UC_SPARC_REG_F58 = 46;
    public static final int UC_SPARC_REG_F60 = 47;
    public static final int UC_SPARC_REG_F62 = 48;
    public static final int UC_SPARC_REG_FCC0 = 49;
    public static final int UC_SPARC_REG_FCC1 = 50;
    public static final int UC_SPARC_REG_FCC2 = 51;
    public static final int UC_SPARC_REG_FCC3 = 52;
    public static final int UC_SPARC_REG_G0 = 53;
    public static final int UC_SPARC_REG_G1 = 54;
    public static final int UC_SPARC_REG_G2 = 55;
    public static final int UC_SPARC_REG_G3 = 56;
    public static final int UC_SPARC_REG_G4 = 57;
    public static final int UC_SPARC_REG_G5 = 58;
    public static final int UC_SPARC_REG_G6 = 59;
    public static final int UC_SPARC_REG_G7 = 60;
    public static final int UC_SPARC_REG_I0 = 61;
    public static final int UC_SPARC_REG_I1 = 62;
    public static final int UC_SPARC_REG_I2 = 63;
    public static final int UC_SPARC_REG_I3 = 64;
    public static final int UC_SPARC_REG_I4 = 65;
    public static final int UC_SPARC_REG_I5 = 66;
    public static final int UC_SPARC_REG_FP = 67;
    public static final int UC_SPARC_REG_I7 = 68;
    public static final int UC_SPARC_REG_ICC = 69;
    public static final int UC_SPARC_REG_L0 = 70;
    public static final int UC_SPARC_REG_L1 = 71;
    public static final int UC_SPARC_REG_L2 = 72;
    public static final int UC_SPARC_REG_L3 = 73;
    public static final int UC_SPARC_REG_L4 = 74;
    public static final int UC_SPARC_REG_L5 = 75;
    public static final int UC_SPARC_REG_L6 = 76;
    public static final int UC_SPARC_REG_L7 = 77;
    public static final int UC_SPARC_REG_O0 = 78;
    public static final int UC_SPARC_REG_O1 = 79;
    public static final int UC_SPARC_REG_O2 = 80;
    public static final int UC_SPARC_REG_O3 = 81;
    public static final int UC_SPARC_REG_O4 = 82;
    public static final int UC_SPARC_REG_O5 = 83;
    public static final int UC_SPARC_REG_SP = 84;
    public static final int UC_SPARC_REG_O7 = 85;
    public static final int UC_SPARC_REG_Y = 86;
    public static final int UC_SPARC_REG_XCC = 87;
    public static final int UC_SPARC_REG_PC = 88;
    public static final int UC_SPARC_REG_PSR = 89;
    public static final int UC_SPARC_REG_ENDING = 90;
    public static final int UC_SPARC_REG_O6 = 84;
    public static final int UC_SPARC_REG_I6 = 67;

}
