// For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT

package unicorn;

public interface PpcConst {

    // PPC CPU

    public static final int UC_CPU_PPC32_401 = 0;
    public static final int UC_CPU_PPC32_401A1 = 1;
    public static final int UC_CPU_PPC32_401B2 = 2;
    public static final int UC_CPU_PPC32_401C2 = 3;
    public static final int UC_CPU_PPC32_401D2 = 4;
    public static final int UC_CPU_PPC32_401E2 = 5;
    public static final int UC_CPU_PPC32_401F2 = 6;
    public static final int UC_CPU_PPC32_401G2 = 7;
    public static final int UC_CPU_PPC32_IOP480 = 8;
    public static final int UC_CPU_PPC32_COBRA = 9;
    public static final int UC_CPU_PPC32_403GA = 10;
    public static final int UC_CPU_PPC32_403GB = 11;
    public static final int UC_CPU_PPC32_403GC = 12;
    public static final int UC_CPU_PPC32_403GCX = 13;
    public static final int UC_CPU_PPC32_405D2 = 14;
    public static final int UC_CPU_PPC32_405D4 = 15;
    public static final int UC_CPU_PPC32_405CRA = 16;
    public static final int UC_CPU_PPC32_405CRB = 17;
    public static final int UC_CPU_PPC32_405CRC = 18;
    public static final int UC_CPU_PPC32_405EP = 19;
    public static final int UC_CPU_PPC32_405EZ = 20;
    public static final int UC_CPU_PPC32_405GPA = 21;
    public static final int UC_CPU_PPC32_405GPB = 22;
    public static final int UC_CPU_PPC32_405GPC = 23;
    public static final int UC_CPU_PPC32_405GPD = 24;
    public static final int UC_CPU_PPC32_405GPR = 25;
    public static final int UC_CPU_PPC32_405LP = 26;
    public static final int UC_CPU_PPC32_NPE405H = 27;
    public static final int UC_CPU_PPC32_NPE405H2 = 28;
    public static final int UC_CPU_PPC32_NPE405L = 29;
    public static final int UC_CPU_PPC32_NPE4GS3 = 30;
    public static final int UC_CPU_PPC32_STB03 = 31;
    public static final int UC_CPU_PPC32_STB04 = 32;
    public static final int UC_CPU_PPC32_STB25 = 33;
    public static final int UC_CPU_PPC32_X2VP4 = 34;
    public static final int UC_CPU_PPC32_X2VP20 = 35;
    public static final int UC_CPU_PPC32_440_XILINX = 36;
    public static final int UC_CPU_PPC32_440_XILINX_W_DFPU = 37;
    public static final int UC_CPU_PPC32_440EPA = 38;
    public static final int UC_CPU_PPC32_440EPB = 39;
    public static final int UC_CPU_PPC32_440EPX = 40;
    public static final int UC_CPU_PPC32_460EXB = 41;
    public static final int UC_CPU_PPC32_G2 = 42;
    public static final int UC_CPU_PPC32_G2H4 = 43;
    public static final int UC_CPU_PPC32_G2GP = 44;
    public static final int UC_CPU_PPC32_G2LS = 45;
    public static final int UC_CPU_PPC32_G2HIP3 = 46;
    public static final int UC_CPU_PPC32_G2HIP4 = 47;
    public static final int UC_CPU_PPC32_MPC603 = 48;
    public static final int UC_CPU_PPC32_G2LE = 49;
    public static final int UC_CPU_PPC32_G2LEGP = 50;
    public static final int UC_CPU_PPC32_G2LELS = 51;
    public static final int UC_CPU_PPC32_G2LEGP1 = 52;
    public static final int UC_CPU_PPC32_G2LEGP3 = 53;
    public static final int UC_CPU_PPC32_MPC5200_V10 = 54;
    public static final int UC_CPU_PPC32_MPC5200_V11 = 55;
    public static final int UC_CPU_PPC32_MPC5200_V12 = 56;
    public static final int UC_CPU_PPC32_MPC5200B_V20 = 57;
    public static final int UC_CPU_PPC32_MPC5200B_V21 = 58;
    public static final int UC_CPU_PPC32_E200Z5 = 59;
    public static final int UC_CPU_PPC32_E200Z6 = 60;
    public static final int UC_CPU_PPC32_E300C1 = 61;
    public static final int UC_CPU_PPC32_E300C2 = 62;
    public static final int UC_CPU_PPC32_E300C3 = 63;
    public static final int UC_CPU_PPC32_E300C4 = 64;
    public static final int UC_CPU_PPC32_MPC8343 = 65;
    public static final int UC_CPU_PPC32_MPC8343A = 66;
    public static final int UC_CPU_PPC32_MPC8343E = 67;
    public static final int UC_CPU_PPC32_MPC8343EA = 68;
    public static final int UC_CPU_PPC32_MPC8347T = 69;
    public static final int UC_CPU_PPC32_MPC8347P = 70;
    public static final int UC_CPU_PPC32_MPC8347AT = 71;
    public static final int UC_CPU_PPC32_MPC8347AP = 72;
    public static final int UC_CPU_PPC32_MPC8347ET = 73;
    public static final int UC_CPU_PPC32_MPC8347EP = 74;
    public static final int UC_CPU_PPC32_MPC8347EAT = 75;
    public static final int UC_CPU_PPC32_MPC8347EAP = 76;
    public static final int UC_CPU_PPC32_MPC8349 = 77;
    public static final int UC_CPU_PPC32_MPC8349A = 78;
    public static final int UC_CPU_PPC32_MPC8349E = 79;
    public static final int UC_CPU_PPC32_MPC8349EA = 80;
    public static final int UC_CPU_PPC32_MPC8377 = 81;
    public static final int UC_CPU_PPC32_MPC8377E = 82;
    public static final int UC_CPU_PPC32_MPC8378 = 83;
    public static final int UC_CPU_PPC32_MPC8378E = 84;
    public static final int UC_CPU_PPC32_MPC8379 = 85;
    public static final int UC_CPU_PPC32_MPC8379E = 86;
    public static final int UC_CPU_PPC32_E500_V10 = 87;
    public static final int UC_CPU_PPC32_E500_V20 = 88;
    public static final int UC_CPU_PPC32_E500V2_V10 = 89;
    public static final int UC_CPU_PPC32_E500V2_V20 = 90;
    public static final int UC_CPU_PPC32_E500V2_V21 = 91;
    public static final int UC_CPU_PPC32_E500V2_V22 = 92;
    public static final int UC_CPU_PPC32_E500V2_V30 = 93;
    public static final int UC_CPU_PPC32_E500MC = 94;
    public static final int UC_CPU_PPC32_MPC8533_V10 = 95;
    public static final int UC_CPU_PPC32_MPC8533_V11 = 96;
    public static final int UC_CPU_PPC32_MPC8533E_V10 = 97;
    public static final int UC_CPU_PPC32_MPC8533E_V11 = 98;
    public static final int UC_CPU_PPC32_MPC8540_V10 = 99;
    public static final int UC_CPU_PPC32_MPC8540_V20 = 100;
    public static final int UC_CPU_PPC32_MPC8540_V21 = 101;
    public static final int UC_CPU_PPC32_MPC8541_V10 = 102;
    public static final int UC_CPU_PPC32_MPC8541_V11 = 103;
    public static final int UC_CPU_PPC32_MPC8541E_V10 = 104;
    public static final int UC_CPU_PPC32_MPC8541E_V11 = 105;
    public static final int UC_CPU_PPC32_MPC8543_V10 = 106;
    public static final int UC_CPU_PPC32_MPC8543_V11 = 107;
    public static final int UC_CPU_PPC32_MPC8543_V20 = 108;
    public static final int UC_CPU_PPC32_MPC8543_V21 = 109;
    public static final int UC_CPU_PPC32_MPC8543E_V10 = 110;
    public static final int UC_CPU_PPC32_MPC8543E_V11 = 111;
    public static final int UC_CPU_PPC32_MPC8543E_V20 = 112;
    public static final int UC_CPU_PPC32_MPC8543E_V21 = 113;
    public static final int UC_CPU_PPC32_MPC8544_V10 = 114;
    public static final int UC_CPU_PPC32_MPC8544_V11 = 115;
    public static final int UC_CPU_PPC32_MPC8544E_V10 = 116;
    public static final int UC_CPU_PPC32_MPC8544E_V11 = 117;
    public static final int UC_CPU_PPC32_MPC8545_V20 = 118;
    public static final int UC_CPU_PPC32_MPC8545_V21 = 119;
    public static final int UC_CPU_PPC32_MPC8545E_V20 = 120;
    public static final int UC_CPU_PPC32_MPC8545E_V21 = 121;
    public static final int UC_CPU_PPC32_MPC8547E_V20 = 122;
    public static final int UC_CPU_PPC32_MPC8547E_V21 = 123;
    public static final int UC_CPU_PPC32_MPC8548_V10 = 124;
    public static final int UC_CPU_PPC32_MPC8548_V11 = 125;
    public static final int UC_CPU_PPC32_MPC8548_V20 = 126;
    public static final int UC_CPU_PPC32_MPC8548_V21 = 127;
    public static final int UC_CPU_PPC32_MPC8548E_V10 = 128;
    public static final int UC_CPU_PPC32_MPC8548E_V11 = 129;
    public static final int UC_CPU_PPC32_MPC8548E_V20 = 130;
    public static final int UC_CPU_PPC32_MPC8548E_V21 = 131;
    public static final int UC_CPU_PPC32_MPC8555_V10 = 132;
    public static final int UC_CPU_PPC32_MPC8555_V11 = 133;
    public static final int UC_CPU_PPC32_MPC8555E_V10 = 134;
    public static final int UC_CPU_PPC32_MPC8555E_V11 = 135;
    public static final int UC_CPU_PPC32_MPC8560_V10 = 136;
    public static final int UC_CPU_PPC32_MPC8560_V20 = 137;
    public static final int UC_CPU_PPC32_MPC8560_V21 = 138;
    public static final int UC_CPU_PPC32_MPC8567 = 139;
    public static final int UC_CPU_PPC32_MPC8567E = 140;
    public static final int UC_CPU_PPC32_MPC8568 = 141;
    public static final int UC_CPU_PPC32_MPC8568E = 142;
    public static final int UC_CPU_PPC32_MPC8572 = 143;
    public static final int UC_CPU_PPC32_MPC8572E = 144;
    public static final int UC_CPU_PPC32_E600 = 145;
    public static final int UC_CPU_PPC32_MPC8610 = 146;
    public static final int UC_CPU_PPC32_MPC8641 = 147;
    public static final int UC_CPU_PPC32_MPC8641D = 148;
    public static final int UC_CPU_PPC32_601_V0 = 149;
    public static final int UC_CPU_PPC32_601_V1 = 150;
    public static final int UC_CPU_PPC32_601_V2 = 151;
    public static final int UC_CPU_PPC32_602 = 152;
    public static final int UC_CPU_PPC32_603 = 153;
    public static final int UC_CPU_PPC32_603E_V1_1 = 154;
    public static final int UC_CPU_PPC32_603E_V1_2 = 155;
    public static final int UC_CPU_PPC32_603E_V1_3 = 156;
    public static final int UC_CPU_PPC32_603E_V1_4 = 157;
    public static final int UC_CPU_PPC32_603E_V2_2 = 158;
    public static final int UC_CPU_PPC32_603E_V3 = 159;
    public static final int UC_CPU_PPC32_603E_V4 = 160;
    public static final int UC_CPU_PPC32_603E_V4_1 = 161;
    public static final int UC_CPU_PPC32_603E7 = 162;
    public static final int UC_CPU_PPC32_603E7T = 163;
    public static final int UC_CPU_PPC32_603E7V = 164;
    public static final int UC_CPU_PPC32_603E7V1 = 165;
    public static final int UC_CPU_PPC32_603E7V2 = 166;
    public static final int UC_CPU_PPC32_603P = 167;
    public static final int UC_CPU_PPC32_604 = 168;
    public static final int UC_CPU_PPC32_604E_V1_0 = 169;
    public static final int UC_CPU_PPC32_604E_V2_2 = 170;
    public static final int UC_CPU_PPC32_604E_V2_4 = 171;
    public static final int UC_CPU_PPC32_604R = 172;
    public static final int UC_CPU_PPC32_740_V1_0 = 173;
    public static final int UC_CPU_PPC32_750_V1_0 = 174;
    public static final int UC_CPU_PPC32_740_V2_0 = 175;
    public static final int UC_CPU_PPC32_750_V2_0 = 176;
    public static final int UC_CPU_PPC32_740_V2_1 = 177;
    public static final int UC_CPU_PPC32_750_V2_1 = 178;
    public static final int UC_CPU_PPC32_740_V2_2 = 179;
    public static final int UC_CPU_PPC32_750_V2_2 = 180;
    public static final int UC_CPU_PPC32_740_V3_0 = 181;
    public static final int UC_CPU_PPC32_750_V3_0 = 182;
    public static final int UC_CPU_PPC32_740_V3_1 = 183;
    public static final int UC_CPU_PPC32_750_V3_1 = 184;
    public static final int UC_CPU_PPC32_740E = 185;
    public static final int UC_CPU_PPC32_750E = 186;
    public static final int UC_CPU_PPC32_740P = 187;
    public static final int UC_CPU_PPC32_750P = 188;
    public static final int UC_CPU_PPC32_750CL_V1_0 = 189;
    public static final int UC_CPU_PPC32_750CL_V2_0 = 190;
    public static final int UC_CPU_PPC32_750CX_V1_0 = 191;
    public static final int UC_CPU_PPC32_750CX_V2_0 = 192;
    public static final int UC_CPU_PPC32_750CX_V2_1 = 193;
    public static final int UC_CPU_PPC32_750CX_V2_2 = 194;
    public static final int UC_CPU_PPC32_750CXE_V2_1 = 195;
    public static final int UC_CPU_PPC32_750CXE_V2_2 = 196;
    public static final int UC_CPU_PPC32_750CXE_V2_3 = 197;
    public static final int UC_CPU_PPC32_750CXE_V2_4 = 198;
    public static final int UC_CPU_PPC32_750CXE_V2_4B = 199;
    public static final int UC_CPU_PPC32_750CXE_V3_0 = 200;
    public static final int UC_CPU_PPC32_750CXE_V3_1 = 201;
    public static final int UC_CPU_PPC32_750CXE_V3_1B = 202;
    public static final int UC_CPU_PPC32_750CXR = 203;
    public static final int UC_CPU_PPC32_750FL = 204;
    public static final int UC_CPU_PPC32_750FX_V1_0 = 205;
    public static final int UC_CPU_PPC32_750FX_V2_0 = 206;
    public static final int UC_CPU_PPC32_750FX_V2_1 = 207;
    public static final int UC_CPU_PPC32_750FX_V2_2 = 208;
    public static final int UC_CPU_PPC32_750FX_V2_3 = 209;
    public static final int UC_CPU_PPC32_750GL = 210;
    public static final int UC_CPU_PPC32_750GX_V1_0 = 211;
    public static final int UC_CPU_PPC32_750GX_V1_1 = 212;
    public static final int UC_CPU_PPC32_750GX_V1_2 = 213;
    public static final int UC_CPU_PPC32_750L_V2_0 = 214;
    public static final int UC_CPU_PPC32_750L_V2_1 = 215;
    public static final int UC_CPU_PPC32_750L_V2_2 = 216;
    public static final int UC_CPU_PPC32_750L_V3_0 = 217;
    public static final int UC_CPU_PPC32_750L_V3_2 = 218;
    public static final int UC_CPU_PPC32_745_V1_0 = 219;
    public static final int UC_CPU_PPC32_755_V1_0 = 220;
    public static final int UC_CPU_PPC32_745_V1_1 = 221;
    public static final int UC_CPU_PPC32_755_V1_1 = 222;
    public static final int UC_CPU_PPC32_745_V2_0 = 223;
    public static final int UC_CPU_PPC32_755_V2_0 = 224;
    public static final int UC_CPU_PPC32_745_V2_1 = 225;
    public static final int UC_CPU_PPC32_755_V2_1 = 226;
    public static final int UC_CPU_PPC32_745_V2_2 = 227;
    public static final int UC_CPU_PPC32_755_V2_2 = 228;
    public static final int UC_CPU_PPC32_745_V2_3 = 229;
    public static final int UC_CPU_PPC32_755_V2_3 = 230;
    public static final int UC_CPU_PPC32_745_V2_4 = 231;
    public static final int UC_CPU_PPC32_755_V2_4 = 232;
    public static final int UC_CPU_PPC32_745_V2_5 = 233;
    public static final int UC_CPU_PPC32_755_V2_5 = 234;
    public static final int UC_CPU_PPC32_745_V2_6 = 235;
    public static final int UC_CPU_PPC32_755_V2_6 = 236;
    public static final int UC_CPU_PPC32_745_V2_7 = 237;
    public static final int UC_CPU_PPC32_755_V2_7 = 238;
    public static final int UC_CPU_PPC32_745_V2_8 = 239;
    public static final int UC_CPU_PPC32_755_V2_8 = 240;
    public static final int UC_CPU_PPC32_7400_V1_0 = 241;
    public static final int UC_CPU_PPC32_7400_V1_1 = 242;
    public static final int UC_CPU_PPC32_7400_V2_0 = 243;
    public static final int UC_CPU_PPC32_7400_V2_1 = 244;
    public static final int UC_CPU_PPC32_7400_V2_2 = 245;
    public static final int UC_CPU_PPC32_7400_V2_6 = 246;
    public static final int UC_CPU_PPC32_7400_V2_7 = 247;
    public static final int UC_CPU_PPC32_7400_V2_8 = 248;
    public static final int UC_CPU_PPC32_7400_V2_9 = 249;
    public static final int UC_CPU_PPC32_7410_V1_0 = 250;
    public static final int UC_CPU_PPC32_7410_V1_1 = 251;
    public static final int UC_CPU_PPC32_7410_V1_2 = 252;
    public static final int UC_CPU_PPC32_7410_V1_3 = 253;
    public static final int UC_CPU_PPC32_7410_V1_4 = 254;
    public static final int UC_CPU_PPC32_7448_V1_0 = 255;
    public static final int UC_CPU_PPC32_7448_V1_1 = 256;
    public static final int UC_CPU_PPC32_7448_V2_0 = 257;
    public static final int UC_CPU_PPC32_7448_V2_1 = 258;
    public static final int UC_CPU_PPC32_7450_V1_0 = 259;
    public static final int UC_CPU_PPC32_7450_V1_1 = 260;
    public static final int UC_CPU_PPC32_7450_V1_2 = 261;
    public static final int UC_CPU_PPC32_7450_V2_0 = 262;
    public static final int UC_CPU_PPC32_7450_V2_1 = 263;
    public static final int UC_CPU_PPC32_7441_V2_1 = 264;
    public static final int UC_CPU_PPC32_7441_V2_3 = 265;
    public static final int UC_CPU_PPC32_7451_V2_3 = 266;
    public static final int UC_CPU_PPC32_7441_V2_10 = 267;
    public static final int UC_CPU_PPC32_7451_V2_10 = 268;
    public static final int UC_CPU_PPC32_7445_V1_0 = 269;
    public static final int UC_CPU_PPC32_7455_V1_0 = 270;
    public static final int UC_CPU_PPC32_7445_V2_1 = 271;
    public static final int UC_CPU_PPC32_7455_V2_1 = 272;
    public static final int UC_CPU_PPC32_7445_V3_2 = 273;
    public static final int UC_CPU_PPC32_7455_V3_2 = 274;
    public static final int UC_CPU_PPC32_7445_V3_3 = 275;
    public static final int UC_CPU_PPC32_7455_V3_3 = 276;
    public static final int UC_CPU_PPC32_7445_V3_4 = 277;
    public static final int UC_CPU_PPC32_7455_V3_4 = 278;
    public static final int UC_CPU_PPC32_7447_V1_0 = 279;
    public static final int UC_CPU_PPC32_7457_V1_0 = 280;
    public static final int UC_CPU_PPC32_7447_V1_1 = 281;
    public static final int UC_CPU_PPC32_7457_V1_1 = 282;
    public static final int UC_CPU_PPC32_7457_V1_2 = 283;
    public static final int UC_CPU_PPC32_7447A_V1_0 = 284;
    public static final int UC_CPU_PPC32_7457A_V1_0 = 285;
    public static final int UC_CPU_PPC32_7447A_V1_1 = 286;
    public static final int UC_CPU_PPC32_7457A_V1_1 = 287;
    public static final int UC_CPU_PPC32_7447A_V1_2 = 288;
    public static final int UC_CPU_PPC32_7457A_V1_2 = 289;
    public static final int UC_CPU_PPC32_ENDING = 290;

    // PPC64 CPU

    public static final int UC_CPU_PPC64_E5500 = 0;
    public static final int UC_CPU_PPC64_E6500 = 1;
    public static final int UC_CPU_PPC64_970_V2_2 = 2;
    public static final int UC_CPU_PPC64_970FX_V1_0 = 3;
    public static final int UC_CPU_PPC64_970FX_V2_0 = 4;
    public static final int UC_CPU_PPC64_970FX_V2_1 = 5;
    public static final int UC_CPU_PPC64_970FX_V3_0 = 6;
    public static final int UC_CPU_PPC64_970FX_V3_1 = 7;
    public static final int UC_CPU_PPC64_970MP_V1_0 = 8;
    public static final int UC_CPU_PPC64_970MP_V1_1 = 9;
    public static final int UC_CPU_PPC64_POWER5_V2_1 = 10;
    public static final int UC_CPU_PPC64_POWER7_V2_3 = 11;
    public static final int UC_CPU_PPC64_POWER7_V2_1 = 12;
    public static final int UC_CPU_PPC64_POWER8E_V2_1 = 13;
    public static final int UC_CPU_PPC64_POWER8_V2_0 = 14;
    public static final int UC_CPU_PPC64_POWER8NVL_V1_0 = 15;
    public static final int UC_CPU_PPC64_POWER9_V1_0 = 16;
    public static final int UC_CPU_PPC64_POWER9_V2_0 = 17;
    public static final int UC_CPU_PPC64_POWER10_V1_0 = 18;
    public static final int UC_CPU_PPC64_ENDING = 19;

    // PPC registers

    public static final int UC_PPC_REG_INVALID = 0;

    // General purpose registers
    public static final int UC_PPC_REG_PC = 1;
    public static final int UC_PPC_REG_0 = 2;
    public static final int UC_PPC_REG_1 = 3;
    public static final int UC_PPC_REG_2 = 4;
    public static final int UC_PPC_REG_3 = 5;
    public static final int UC_PPC_REG_4 = 6;
    public static final int UC_PPC_REG_5 = 7;
    public static final int UC_PPC_REG_6 = 8;
    public static final int UC_PPC_REG_7 = 9;
    public static final int UC_PPC_REG_8 = 10;
    public static final int UC_PPC_REG_9 = 11;
    public static final int UC_PPC_REG_10 = 12;
    public static final int UC_PPC_REG_11 = 13;
    public static final int UC_PPC_REG_12 = 14;
    public static final int UC_PPC_REG_13 = 15;
    public static final int UC_PPC_REG_14 = 16;
    public static final int UC_PPC_REG_15 = 17;
    public static final int UC_PPC_REG_16 = 18;
    public static final int UC_PPC_REG_17 = 19;
    public static final int UC_PPC_REG_18 = 20;
    public static final int UC_PPC_REG_19 = 21;
    public static final int UC_PPC_REG_20 = 22;
    public static final int UC_PPC_REG_21 = 23;
    public static final int UC_PPC_REG_22 = 24;
    public static final int UC_PPC_REG_23 = 25;
    public static final int UC_PPC_REG_24 = 26;
    public static final int UC_PPC_REG_25 = 27;
    public static final int UC_PPC_REG_26 = 28;
    public static final int UC_PPC_REG_27 = 29;
    public static final int UC_PPC_REG_28 = 30;
    public static final int UC_PPC_REG_29 = 31;
    public static final int UC_PPC_REG_30 = 32;
    public static final int UC_PPC_REG_31 = 33;
    public static final int UC_PPC_REG_CR0 = 34;
    public static final int UC_PPC_REG_CR1 = 35;
    public static final int UC_PPC_REG_CR2 = 36;
    public static final int UC_PPC_REG_CR3 = 37;
    public static final int UC_PPC_REG_CR4 = 38;
    public static final int UC_PPC_REG_CR5 = 39;
    public static final int UC_PPC_REG_CR6 = 40;
    public static final int UC_PPC_REG_CR7 = 41;
    public static final int UC_PPC_REG_FPR0 = 42;
    public static final int UC_PPC_REG_FPR1 = 43;
    public static final int UC_PPC_REG_FPR2 = 44;
    public static final int UC_PPC_REG_FPR3 = 45;
    public static final int UC_PPC_REG_FPR4 = 46;
    public static final int UC_PPC_REG_FPR5 = 47;
    public static final int UC_PPC_REG_FPR6 = 48;
    public static final int UC_PPC_REG_FPR7 = 49;
    public static final int UC_PPC_REG_FPR8 = 50;
    public static final int UC_PPC_REG_FPR9 = 51;
    public static final int UC_PPC_REG_FPR10 = 52;
    public static final int UC_PPC_REG_FPR11 = 53;
    public static final int UC_PPC_REG_FPR12 = 54;
    public static final int UC_PPC_REG_FPR13 = 55;
    public static final int UC_PPC_REG_FPR14 = 56;
    public static final int UC_PPC_REG_FPR15 = 57;
    public static final int UC_PPC_REG_FPR16 = 58;
    public static final int UC_PPC_REG_FPR17 = 59;
    public static final int UC_PPC_REG_FPR18 = 60;
    public static final int UC_PPC_REG_FPR19 = 61;
    public static final int UC_PPC_REG_FPR20 = 62;
    public static final int UC_PPC_REG_FPR21 = 63;
    public static final int UC_PPC_REG_FPR22 = 64;
    public static final int UC_PPC_REG_FPR23 = 65;
    public static final int UC_PPC_REG_FPR24 = 66;
    public static final int UC_PPC_REG_FPR25 = 67;
    public static final int UC_PPC_REG_FPR26 = 68;
    public static final int UC_PPC_REG_FPR27 = 69;
    public static final int UC_PPC_REG_FPR28 = 70;
    public static final int UC_PPC_REG_FPR29 = 71;
    public static final int UC_PPC_REG_FPR30 = 72;
    public static final int UC_PPC_REG_FPR31 = 73;
    public static final int UC_PPC_REG_LR = 74;
    public static final int UC_PPC_REG_XER = 75;
    public static final int UC_PPC_REG_CTR = 76;
    public static final int UC_PPC_REG_MSR = 77;
    public static final int UC_PPC_REG_FPSCR = 78;
    public static final int UC_PPC_REG_CR = 79;
    public static final int UC_PPC_REG_ENDING = 80;

}
