// For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT

package unicorn;

public interface M68kConst {

    // M68K CPU

    public static final int UC_CPU_M68K_M5206 = 0;
    public static final int UC_CPU_M68K_M68000 = 1;
    public static final int UC_CPU_M68K_M68020 = 2;
    public static final int UC_CPU_M68K_M68030 = 3;
    public static final int UC_CPU_M68K_M68040 = 4;
    public static final int UC_CPU_M68K_M68060 = 5;
    public static final int UC_CPU_M68K_M5208 = 6;
    public static final int UC_CPU_M68K_CFV4E = 7;
    public static final int UC_CPU_M68K_ANY = 8;
    public static final int UC_CPU_M68K_ENDING = 9;

    // M68K registers

    public static final int UC_M68K_REG_INVALID = 0;
    public static final int UC_M68K_REG_A0 = 1;
    public static final int UC_M68K_REG_A1 = 2;
    public static final int UC_M68K_REG_A2 = 3;
    public static final int UC_M68K_REG_A3 = 4;
    public static final int UC_M68K_REG_A4 = 5;
    public static final int UC_M68K_REG_A5 = 6;
    public static final int UC_M68K_REG_A6 = 7;
    public static final int UC_M68K_REG_A7 = 8;
    public static final int UC_M68K_REG_D0 = 9;
    public static final int UC_M68K_REG_D1 = 10;
    public static final int UC_M68K_REG_D2 = 11;
    public static final int UC_M68K_REG_D3 = 12;
    public static final int UC_M68K_REG_D4 = 13;
    public static final int UC_M68K_REG_D5 = 14;
    public static final int UC_M68K_REG_D6 = 15;
    public static final int UC_M68K_REG_D7 = 16;
    public static final int UC_M68K_REG_SR = 17;
    public static final int UC_M68K_REG_PC = 18;
    public static final int UC_M68K_REG_CR_SFC = 19;
    public static final int UC_M68K_REG_CR_DFC = 20;
    public static final int UC_M68K_REG_CR_VBR = 21;
    public static final int UC_M68K_REG_CR_CACR = 22;
    public static final int UC_M68K_REG_CR_TC = 23;
    public static final int UC_M68K_REG_CR_MMUSR = 24;
    public static final int UC_M68K_REG_CR_SRP = 25;
    public static final int UC_M68K_REG_CR_USP = 26;
    public static final int UC_M68K_REG_CR_MSP = 27;
    public static final int UC_M68K_REG_CR_ISP = 28;
    public static final int UC_M68K_REG_CR_URP = 29;
    public static final int UC_M68K_REG_CR_ITT0 = 30;
    public static final int UC_M68K_REG_CR_ITT1 = 31;
    public static final int UC_M68K_REG_CR_DTT0 = 32;
    public static final int UC_M68K_REG_CR_DTT1 = 33;
    public static final int UC_M68K_REG_ENDING = 34;

}
