// For Unicorn Engine. AUTO-GENERATED FILE, DO NOT EDIT

namespace UnicornEngine.Const

open System

[<AutoOpen>]
module M68k =

    // M68K CPU

    let UC_CPU_M68K_M5206 = 0
    let UC_CPU_M68K_M68000 = 1
    let UC_CPU_M68K_M68020 = 2
    let UC_CPU_M68K_M68030 = 3
    let UC_CPU_M68K_M68040 = 4
    let UC_CPU_M68K_M68060 = 5
    let UC_CPU_M68K_M5208 = 6
    let UC_CPU_M68K_CFV4E = 7
    let UC_CPU_M68K_ANY = 8
    let UC_CPU_M68K_ENDING = 9

    // M68K registers

    let UC_M68K_REG_INVALID = 0
    let UC_M68K_REG_A0 = 1
    let UC_M68K_REG_A1 = 2
    let UC_M68K_REG_A2 = 3
    let UC_M68K_REG_A3 = 4
    let UC_M68K_REG_A4 = 5
    let UC_M68K_REG_A5 = 6
    let UC_M68K_REG_A6 = 7
    let UC_M68K_REG_A7 = 8
    let UC_M68K_REG_D0 = 9
    let UC_M68K_REG_D1 = 10
    let UC_M68K_REG_D2 = 11
    let UC_M68K_REG_D3 = 12
    let UC_M68K_REG_D4 = 13
    let UC_M68K_REG_D5 = 14
    let UC_M68K_REG_D6 = 15
    let UC_M68K_REG_D7 = 16
    let UC_M68K_REG_SR = 17
    let UC_M68K_REG_PC = 18
    let UC_M68K_REG_CR_SFC = 19
    let UC_M68K_REG_CR_DFC = 20
    let UC_M68K_REG_CR_VBR = 21
    let UC_M68K_REG_CR_CACR = 22
    let UC_M68K_REG_CR_TC = 23
    let UC_M68K_REG_CR_MMUSR = 24
    let UC_M68K_REG_CR_SRP = 25
    let UC_M68K_REG_CR_USP = 26
    let UC_M68K_REG_CR_MSP = 27
    let UC_M68K_REG_CR_ISP = 28
    let UC_M68K_REG_CR_URP = 29
    let UC_M68K_REG_CR_ITT0 = 30
    let UC_M68K_REG_CR_ITT1 = 31
    let UC_M68K_REG_CR_DTT0 = 32
    let UC_M68K_REG_CR_DTT1 = 33
    let UC_M68K_REG_ENDING = 34

